@@ -1,4 +1,5 @@
 from logging import Logger
+from subprocess import Popen
 
 from supbot.app_driver import AppDriver
 from supbot.system import System
@@ -7,3 +8,4 @@ logger: Logger
 kwargs = None
 system: System
 driver: AppDriver
+appium_process: Popen
