@@ -42,11 +42,13 @@ class System:
                 msg = self.format(record)
                 print(msg)
 
+        self.verbose = "verbose" in g.kwargs and g.kwargs["verbose"] is not None
+
         logging.getLogger("selenium").setLevel(logging.ERROR)
         logging.getLogger("urllib3").setLevel(logging.ERROR)
         logging.getLogger("appium").setLevel(logging.DEBUG)
 
-        logging.basicConfig(level=logging.DEBUG)
+        logging.basicConfig(level=logging.DEBUG if self.verbose else logging.INFO)
 
         logging.getLogger().handlers = []
 
@@ -56,14 +58,21 @@ class System:
         handler.setFormatter(logging.Formatter(fmt=log_format))
         g.logger.addHandler(handler)
 
-        appium_logs = logging.getLogger('appium')
-        fh = logging.FileHandler('appium.log', encoding='utf-8')
-        appium_logs.addHandler(fh)
+        if self.verbose:
+            appium_logs = logging.getLogger('appium')
+            fh = logging.FileHandler('appium.log', encoding='utf-8')
+            appium_logs.addHandler(fh)
 
+        """
+        1: not started
+        2: looper started
+        0: quit requested
+        -1: looper finished
+        -2: appium finished
+        """
         self.status = 1
+
         self._action_buffer: ActionBuffer = {}
-        # todo remove action archive (no need of it)
-        self.action_achieve: ActionBuffer = {}
         self._looper_thread = threading.Thread(target=looper.start)
         self._supbot = supbot
         g.system = self
@@ -92,6 +101,8 @@ class System:
         looper thread finishes when,  `_status` flag is False and there are no actions left in `_action_buffer`
         """
         self._looper_thread.join()
+        if g.driver.appium_thread is not None:
+            g.driver.appium_thread.join()
 
     def quit(self):
         """
